/*
 _________________________
         Shrub.inc
 ~for POV-Ray version 3.7~
     2011 Samuel Benge
 
 
 Makes a basic shrub...
 
 3.7 features:
  +radiosity sampling importance 0.2
  +backside illumination
 
 Outside Dependencies:
  1) UnderDish.inc
  2) FlowerPot.inc
 
 __________________________
 Potted_Shrub(TreeSeed, Seg)
  *TreeSeed = a random seed (eg. seed(1019))
  *Seg      = number of segments, controls length, #branches (good values: between 50-170)

*/
 
*/

#include "UnderDish.inc"
#include "FlowerPot.inc"

#macro Shrub_Lerp(A,B,C) A*(1-C)+B*C #end

#declare Shrub_Leaf =
 sphere{0,1
  clipped_by{
   sphere{-z,1 scale<.2,.5,1>}
  }
  normal{planar -.5 scallop_wave rotate z*90 scale .2}
  finish{specular .2 metallic .5 diffuse .75,.25 roughness 0.07}
  translate <0,.5,.865>
 }

#macro Set_Leaf() // leaf instance
 #if(V>Segments*.25) // leaf appear on upper levels
  #if(rand(ShrubSeed)>.5 | V>Segments-5) /* average 1 leaf per-segment, force upper levels */
   object{
    Shrub_Leaf
    pigment{rgb<.03,.27+.5*pow(V/Segments,2),.1>} // pigment instance
    translate y*max(Rad*Rad_Actual,.01)
    rotate x*( 90 + 40*(1-2*rand(ShrubSeed)) )
    rotate y*720*rand(ShrubSeed)
    scale Rad_Actual/1
    translate Pos
   }
  #end
 #end
#end

#macro Shrub_Branching(Pos,Norm,Rad,Rad_Actual,Iter,Segments)
 
 #if(Iter>0)
  
  #local V = 0;
  #while(V<Segments)
   
   #local X = Pos.x; #local Y = Pos.y; #local Z = Pos.z;
   
   #local Norm = vnormalize( Shrub_Lerp(Norm, <0,-pow(V/Segments,2),0>+1.25*(1-2*<rand(ShrubSeed),rand(ShrubSeed),rand(ShrubSeed)>), 0.075));
   
   #local RadOld = Rad;
   #local Rad = Rad*(1-2/Segments);
   
   #local PosOld = Pos;
   #local Pos = Pos + Norm*(.03+.25*Rad);
   
   // branches
   sphere{Pos,max(Rad*Rad_Actual,.01)}
   #if(vlength(PosOld-Pos) != 0)
    cone{PosOld, max(RadOld*Rad_Actual,.01), Pos, max(Rad*Rad_Actual,.01)}
   #end
   
   Set_Leaf()
   
   #if(mod(V,20)=2)
    #local PosNew = Pos;
    
    #local NormNew = vnormalize(Norm+V*.1*(<1,0,1>-2*<rand(ShrubSeed),0,rand(ShrubSeed)>)+.5*(rand(ShrubSeed)-.25));
    
    #local RadNew = Rad*.75;
    #local IterNew = Iter;
    #local SegmentsNew = Segments-V;
    
    Shrub_Branching(PosNew, NormNew, RadNew, Rad_Actual, IterNew-1, SegmentsNew+20)
    
   #end
   
   #local V = V + 1;
  
  #end   // while
  
 #end  // Iter>0
 
#end // Shrub_Branching

#macro Potted_Shrub(ShrubSeed, Seg)
 
 #local Shrub_Noise = function{ pattern{ bumps translate z*rand(ShrubSeed)*1000 scale 2} }
 
 union{
  
  // the shrub
  union{
   Shrub_Branching(
    <0,.5,0>, // initial position
    <0,1,0>,  // initial direction (hanging +x = <.5,1,0>)
    .2,       // length
    .5,       // radius (length*radius = branch thickness)
    4,        // number of iterations
    Seg       // number of segments (affects length and branch count)
   )
   pigment{rgb<.9,.6,.5>} // branch pigment
   radiosity{importance 0.2}
  }
  
  // pot
  union{
   object{UnderDish}
   object{FlowerPot}
   scale .75
   pigment{rgb<.6,.25,.15>}
   normal{granite .04 scale<30,.1,30> no_bump_scale translate y*rand(ShrubSeed)*100}
  }
  
 }

#end